/*	lemon_board Copyright (C) 2022 Wael Karram.
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU Affero General Public License as published by
*	the Free Software Foundation, version 3 of the License.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU Affero General Public License for more details.
*
*	You should have received a copy of the GNU Affero General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

/* Function definitions. */
/* This function attempts to open and read a file. */
/* Returns NULL on failure. */
/* Caller must free returned string. */
static inline char *read_file(const char *restrict pathname, size_t *length) {
	/* Sanity check. */
	assert(pathname);

	/* Local variables. */
	FILE *input;
	char *buffer;

	/* Set the locale. */
	setlocale(LC_ALL, LOCALE_EN_US_UTF8_FLAT_FILE_UTILS_H);

	/* Attempt to open the file. */
	input = fopen(pathname, FILE_READ_ONLY_FLAT_FILE_UTILS_H);

	/* Check if successful. */
	if (!input) {
		*length = 0L;
		return NULL;
	}

	/* TODO: rewrite to handle potential fclose errors!!! */

	/* Attempt to read the whole buffer. */
	if (getdelim(&buffer, length, '\0', input) == -1) {
		free(buffer);
		fclose(input);
		return NULL;
	}
	fclose(input);
	return buffer;
}

/* This function attempts to open/create and write to a file. */
/* Returns WRITE_SUCCESS_FLAT_FILE_UTILS_H on success. */
/* Returns OPEN_ERROR_FLAT_FILE_UTILS_H when unable to open the file. */
static inline int write_to_file(const char *restrict pathname, const char *restrict string) {
	/* Sanity check. */
	assert(pathname && string);

	/* Local variables. */
	FILE *output;

	/* Set the locale. */
	setlocale(LC_ALL, LOCALE_EN_US_UTF8_FLAT_FILE_UTILS_H);

	/* Attempt to open the file, in RW mode. */
	output = fopen(pathname, FILE_READ_WRITE_FLAT_FILE_UTILS_H);

	/* Check if successful. */
	if (!output)
		return OPEN_ERROR_FLAT_FILE_UTILS_H;

	/* Write to the file. */
	fputs(output, string);

	/* Close the file. */
	fclose(output);
	return WRITE_SUCCESS_FLAT_FILE_UTILS_H;
}

/* This function attempts to open/create and append to a file. */
static inline int append_to_file(const char *restrict pathname, const char *restrict string) {
	/* Sanity check. */
	assert(pathname && string);

	/* Local variables. */
	FILE *output;

	/* Set the locale. */
	setlocale(LC_ALL, LOCALE_EN_US_UTF8_FLAT_FILE_UTILS_H);

	/* Attempt to open the file, in append mode. */
	output = fopen(pathname, FILE_APPEND_ONLY_FLAT_FILE_UTILS_H);

	/* Check if successful. */
	if (!output)
		return OPEN_ERROR_FLAT_FILE_UTILS_H;

	/* Write to the file. */
	fputs(output, string);

	/* Close the file. */
	fclose(output);
	return WRITE_SUCCESS_FLAT_FILE_UTILS_H;
}
