/*	lemon_board Copyright (C) 2022 Wael Karram.
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the Affero GNU General Public License as published by
*	the Free Software Foundation, version 3 of the License.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU Affero General Public License for more details.
*
*	You should have received a copy of the GNU Affero General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

/* Include guard. */
#ifndef QUERY_H
#define QUERY_H

/* Includes. */
#include "post.h"
#include "thread.h"
#include "board.h"

/* Define functions to query different struct types. */
/* This function queries a post by the ID thereof. */
/* Takes the post ID and path of the thread to be passed to the supplied per-post query function, with an optional void* arg. */
static inline post_t *query_post_by_id(const size_t post_id, const char *thread_path, void *opt, void* (*query_function)(const size_t, const char*, const void*));
/* Takes the OP ID and path of the thread to be passed to the supplied per-post query function, with an optional void* arg. */
static inline thread_t *query_thread_by_id(const size_t op_id, const char *thread_path, void *opt, void* (*query_function)(const size_t, const char*, const void*));
/* Takes the name of the board to be passed to the supplied per-board query function, with an optional void* arg. */
static inline board_t *query_board(const char *board_name, void *opt, void* (*query_function)(const char*, const void*));
#endif	/* QUERY_H */
