/*	lemon_board Copyright (C) 2022 Wael Karram.
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU Affero General Public License as published by
*	the Free Software Foundation, version 3 of the License.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU Affero General Public License for more details.
*
*	You should have received a copy of the GNU Affero General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

/* Include guard. */
#ifndef CORE_H
#define CORE_H

/* Include headers. */
/* Include header for parsing arguements. */
#include <argp.h>
/* Include stdlib for allocations. */
#include <stdlib.h>
/* Include various headers for daemonizing. */
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <syslog.h>
/* Definitions for debugging and header for assertions. */
#ifdef DEBUG
#ifndef NDEBUG
#define	NDEBUG
#endif	/* NDEBUG */
#endif	/* DEBUG */
#include <assert.h>
/* Header for config reading, parsing and writing. */
#include "config/config.h"

/* Define constants. */
#define TRUE				1
#define FALSE				0
#define ARG_NUM				1
#define LOG_NAME_DAEMON			"lemon_boardd"
#define LOG_START			"Started daemon."
#define LOG_TERMINATE			"Terminated daemon."
#define LOG_CONFIG_RELOAD_FAIL		"Failed to reload config."
#define LOG_CONFIG_RELOAD_SUCCESS	"Successfully reloaded config."

/* Text constants. */
const char *config_file_error	=	"Error reading or parsing config!\n";	/* Printed on config file error. */

/* Input parser. */
const char *argp_program_version =	"lemon_board 0.1";			/* Program version. */
const char *arpg_program_bug_address =	"<placeholder@waelk.tech>"		/* Bug reports. */
static char doc[] = "Lemon Board -- an imageboard software written in C";	/* Program Documentation. */
static char args_doc[] = "ARG1 ARG2";						/* A description of accepted arguments. */
static struct argp_option options[] = {
	{"verbose",	'v', 0,		0,	"Produce verbose output"			},
	{"quiet",	'q', 0,		0,	"Don't produce any output"			},
	{"silent",	's', 0,		OPTION_ALIAS						},
	{"output",	'o', "FILE",	0,	"Output to FILE instead of standard output"	},
	{	0										}
};										/* The options the program understands. */
struct arguments {
	char *args[2];		/* arg1 and arg2. */
	int silent, verbose;
	char *output_file;
};										/* Used by main to communicate with parse_opt. */
static error_t parse_opt(int key, char *arg, struct argp_state *state);		/* Declare parser function. */
static struct argp argp;							/* Declare struct variable. */

/* Configuration struct pointer. */
static config_t *config;

/* Define functions. */
/* Function to handle daemonization. */
static void daemonize(config_t *config);
/* Function to reload config. */
static void config_reload(config_t **config, char *path);

#endif	/* CORE_H */
