/*	lemon_board Copyright (C) 2022 Wael Karram.
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU Affero General Public License as published by
*	the Free Software Foundation, version 3 of the License.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU Affero General Public License for more details.
*
*	You should have received a copy of the GNU Affero General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

/* Include guard. */
#ifndef GEMTEXT_GENERATOR_H
#define GEMTEXT_GENERATOR_H

/* Include query.h */
#include "../../backend/query.h"
/* Include strapp.h */
#include "../../C_lib/strings/strapp.h"

/* Setup debug if needed. */
#ifdef DEBUG
#ifndef NDEBUG
#define NDEBUG
#endif /* NDEBUG */
#endif /* DEBUG */
/* Include assert.h to allow code generation. */
#include <assert.h>
/* Include stdio.h for *prinf. */
#include <stdio.h>

/* Define constants. */
#define GEMTEXT_GENERATOR_H_ID_BUFFER_SIZE	256L

/* Define structs. */
typedef struct Gemtext_Post {
	/* Post text. */
	char *text;
	/* Text length. */
	size_t length;
	/* Text allocation length. */
	size_t allocated_length;
} gemtext_post_t;

/* Define functions. */
/* This function is used to generate one post. */
static inline gemtext_post_t *generate_post_gemtext(post_t *post);
/* This function is used to generate a thread. */
static inline char *generate_thread_gemtext(thread_t *thread);
/* This function is used to generate a board. */
static inline char *generate_board_gemtext(board_t *board, size_t max_posts);
/* This function is used to generate an index. */
static inline char *generate_index_gemtext(index_t *index);

/* Helper functions. */
/* This function frees a gemtext_post_t struct. */
static inline void free_gemtext_post_t(gemtext_post_t *post);
/* This function frees an array of gemtext_post_t structs. */
static inline void free_gemtext_post_t_array(gemtext_post_t **array, size_t length);
/* This function is used to generate a thread's preview. */
static inline char *generate_thread_preview_gemtext(thread_t *thread, size_t max_posts);
/* This function frees strings in an array. */
static inline void free_strings_array(char **array, size_t length);

#endif	/* GEMTEXT_GENERATOR_H */
