/*	lemon_board Copyright (C) 2022 Wael Karram.
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU Affero General Public License as published by
*	the Free Software Foundation, version 3 of the License.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU Affero General Public License for more details.
*
*	You should have received a copy of the GNU Affero General Public License
*	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

/* Include the header. */
#include "string_util.h"

/* Function implementations. */
/* This function is used to calculate the string length for a given value. */
static inline size_t ulltostrlen(const unsigned long long value) {
	return (size_t)((ciel(log10(value)) + 1L) * sizeof(char));
}

/* This function converts an unsigned long long into a string, caller should de-allocate - returns NULL on failure. */
static inline char* ulltostr(const unsigned long long value) {
	/* Local variables. */
	size_t length;
	char *tmp, *buffer;

	/* Attempt to allocate the buffer. */
	buffer = (char*) malloc(ulltostrlen(value));

	/* Check if the allocation failed. */
	if (!buffer)
		return NULL;


	/* Convert. */
	sprintf(buffer, CONVERSION_FORMAT_STRING_UTIL_H, value);

	/* Calculate the length. */
	length = strlen(buffer);

	/* Check if there is a need to resize. */
	if (length < ULLONG_MAX) {
		tmp = realloc(buffer, (length + 1L));
		buffer = (tmp) ? tmp : buffer;
	}

	/* Return the resultant buffer. */
	return buffer;
}
